package org.communiquons.dccaggregator.activities

import android.app.AlertDialog
import android.os.Bundle
import android.view.ContextMenu
import android.view.MenuItem
import android.view.View
import android.widget.AdapterView
import android.widget.Toast
import com.journeyapps.barcodescanner.ScanContract
import com.journeyapps.barcodescanner.ScanOptions
import org.communiquons.dccaggregator.R
import org.communiquons.dccaggregator.WALLET_URL
import org.communiquons.dccaggregator.adapters.CertificatesAdapter
import org.communiquons.dccaggregator.databinding.ActivityCertsManagerBinding
import org.communiquons.dccaggregator.models.Certificate
import java.net.URLDecoder
import java.nio.charset.StandardCharsets

class CertsManager : BaseActivity() {

    private lateinit var binding: ActivityCertsManagerBinding

    private val certsList by lazy {
        ArrayList(certsManager.certsList)
    }

    private val listAdapter by lazy {
        CertificatesAdapter(this, certsList)
    }

    private val TAG = CertsManager::class.java.canonicalName

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityCertsManagerBinding.inflate(layoutInflater)
        setContentView(binding.root)

        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        // Initialize certificates list
        binding.certsList.adapter = listAdapter
        registerForContextMenu(binding.certsList)

        binding.scanCertButton.setOnClickListener {
            addNewCertificate()
        }

        intent?.dataString.toString().apply {
            if (this.startsWith(WALLET_URL))
                processGivenCertificate(this)
        }

        refreshList()
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            finish()
            return true
        }

        return super.onOptionsItemSelected(item)
    }

    private val barcodeLauncher = registerForActivityResult(ScanContract()) {
        if (it.contents == null)
            return@registerForActivityResult

        processGivenCertificate(it.contents)
    }

    private fun addNewCertificate() {
        barcodeLauncher.launch(ScanOptions().apply {
            setOrientationLocked(true)
            setDesiredBarcodeFormats(ScanOptions.QR_CODE)
            setPrompt(resources.getString(R.string.scan_qr_prompt))
        })
    }

    private fun processGivenCertificate(input: String) {
        // Decode input if required
        val decoded = if (input.startsWith(WALLET_URL)) {
            URLDecoder.decode(input.replace(WALLET_URL, ""), StandardCharsets.UTF_8.displayName())
        } else {
            input
        }

        try {
            certsManager.addCertificate(Certificate(decoded))

            refreshList()
        } catch (e: Exception) {
            e.printStackTrace()
            Toast.makeText(this, R.string.invalid_certificate, Toast.LENGTH_SHORT).show()
            return
        }
    }

    private fun refreshList() {
        certsList.clear()
        certsList.addAll(certsManager.certsList)

        listAdapter.notifyDataSetChanged()
    }

    override fun onCreateContextMenu(
        menu: ContextMenu?,
        v: View?,
        menuInfo: ContextMenu.ContextMenuInfo?
    ) {
        super.onCreateContextMenu(menu, v, menuInfo)
        if (v == binding.certsList)
            menuInflater.inflate(R.menu.cert_menu, menu)
    }

    override fun onContextItemSelected(item: MenuItem): Boolean {

        if (item.itemId == R.id.delete_cert) {
            val pos = (item.menuInfo as AdapterView.AdapterContextMenuInfo).position
            confirmDeleteCertificate(certsList.get(pos))
            return true
        }

        return super.onContextItemSelected(item)
    }

    private fun confirmDeleteCertificate(cert: Certificate) {
        AlertDialog.Builder(this).apply {
            setTitle(R.string.remove_cert)
            setMessage(R.string.msg_confirm_remove_cert)
            setNegativeButton(R.string.no, null)
            setPositiveButton(R.string.delete_cert) { _, _ ->
                certsManager.removeCertificate(cert)
                refreshList()
            }
            setIcon(R.drawable.ic_warning)
            show()
        }
    }
}